#include <stdio.h>
#include <stdlib.h>
#include <time.h>

int main() {
    double balance = 10000, kellyBalance = 10000;
    double winProb, rr, thisBet, thisKellyBet;
    int res = 0, nBets, i, j;
    char buf[51];

    printf("This program is aimed to demonstrate the performance of the Kelly criterion ");
    printf("in a series of bets. According to Mr. Kelly, his criterion performs asyntotically better ");
    printf("than any other substantially different betting strategy.\n\n");

    printf("You'll be performing a series of bets, each with the same winning probability. ");
    printf("For every bet that you make, you'll see its outcome, your current balance, ");
    printf("and the balance of Mr. Kelly. To make things even more evident, Mr. Kelly will ");
    printf("only win when you win and lose when you lose!\n\n");

    while(1) {
        printf("How many bets do you want to make? (Mr. Kelly gets very good with n>=50): ");
        fgets(buf, 50, stdin);
        res = sscanf(buf, "%d", &nBets);
        if(res != 1 || nBets <= 0)
            printf("Now, that's not a valid number and you know it! Try again, will ya?\n\n");
        else
            break;
    }

    while(1) {
        printf("\nWhat do you want your winning probability to be? (1-99) ");
        fgets(buf, 50, stdin);
        res = sscanf(buf, "%lf", &winProb);
        if(res != 1 || winProb < 1 || winProb > 99)
            printf("Now, that's not a valid number and you know it! Try again, will ya?\n\n");
        else
            break;
    }

    printf("\nA reward-to-risk ratio is the ratio between the possible win and the possible loss. ");
    printf("For instance, a ratio of 2 means that if you win your bet, you win twice ");
    printf("your initial bet (if you lose, you always lose your bet, no more and no less). ");
    printf("A R-to-R ratio can be < 1, but not <= 0. Be advised that if Mr. Kelly thinks ");
    printf("the betting conditions are unfavorable, he won't bet at all!\n\n");

    while(1) {
        printf("What do you want your reward-to-risk ratio to be? ");

        fgets(buf, 50, stdin);
        res = sscanf(buf, "%lf", &rr);
        if(res != 1 || rr <= 0)
            printf("Now, that's not a valid number and you know it! Try again, will ya?\n");
        else
            break;
    }

    printf("\nAlrighty then! You selected:\nNum bets: %d\tWinning probability: %.3f\tReward/Risk: %.3f\n",
        nBets, winProb, rr);

    srand(time(NULL));
    for(i = 1; i <= nBets && balance >= 0.01; i++) {
        while(1) {
            printf("\nBet #%d/%d (You: $%.2f, Mr. Kelly: $%.2f)\n", i, nBets, balance, kellyBalance);
            printf("How much would you like to bet this time? ");

            fgets(buf, 50, stdin);
            res = sscanf(buf, "%lf", &thisBet);
            if(res != 1 || thisBet <= 0 || thisBet > balance)
                printf("Now, that's not a valid number and you know it! Try again, will ya?\n");
            else
                break;
        }

        thisKellyBet = kellyBalance*((rr*winProb/100-(1-winProb/100))/rr);
        if(thisKellyBet < 0)
            thisKellyBet = 0;
        printf("Mr. Kelly chose to bet $%.2f\n", thisKellyBet);
        if(rand() < RAND_MAX*winProb/100) {
            balance += thisBet*rr;
            kellyBalance += thisKellyBet*rr;
        } else {
            balance -= thisBet;
            kellyBalance -= thisKellyBet;
        }
    }

    if(i < nBets+1) {
        printf("\nHA! Sounds like you're broke... this is how Mr. Kelly has performed at the end of the series...\n");
        for(j = i; j <= nBets; j++) {
            thisKellyBet = kellyBalance*((rr*winProb/100-(1-winProb/100))/rr);
            if(rand() < RAND_MAX*winProb/100) {
                kellyBalance += thisKellyBet*rr;
            } else {
                kellyBalance -= thisKellyBet*rr;
            }
        }
    }

    printf("\nFinal balance: $%.2f\tFinal Kelly balance: $%.2f\n\n", balance, kellyBalance);

    if(balance > kellyBalance)
        printf("Looks like you won this time! But try increasing n and see how it goes...\n\n");
    else {
        printf("Looks like Mr. Kelly won again! What did I tell you?\n");
        printf("Mr. Kelly always bets:\n\n\tcurrentBalance * (R-to-R*winProb - (1-winProb)) / R-to-R\n\n");
        printf("as long as the expected value\n\n");
        printf("\tR-to-R*winProb - (1-winProb) > 0.\n\n");
    }

    printf("Press ENTER to close this window... ");
    scanf("%c", &i);

    return 0;
}
